# -*- coding: utf-8 -*-
"""
copyright 2024 Peak Design
this file is part of hsr1, which is distributed under the GNU Lesser General Public License v3 (LGPL)
"""
import os

import sqlite3

from hsr1.db import (SqliteDBLoad,
                     Serialisation)



class SqliteLoadRaw():
    def __init__(self, db_name:str):
        if os.path.isfile(db_name):
            self.exists = True
        else:
            self.exists = False
        self.db_name = db_name
    
    
    def load(self, columns:[str]=[], 
             start_time=None, 
             end_time=None):
        
        if columns == []:
            columns = self.load_table_names()
            
            if not "raw_data" in columns:
                raise ValueError("this database dosent have raw_data")
            
            columns = columns["raw_data"]
        
        
        if not os.path.isfile(self.db_name):
            raise ValueError(f"database '{self.db_name}' does not exist")
        
        time_col = "pc_time_end_measurement"
        
        time_condition = " WHERE "
        if start_time != None:
            times_after = time_col+" > \""+str(start_time)+"\" and "
            time_condition += times_after
        
        if end_time != None:
            times_before =  time_col+" < \""+str(end_time)+"\" and "
            time_condition += times_before
        
        if time_condition == " WHERE ":
            time_condition = ""
        
        sql = "SELECT "+", ".join(columns)+" FROM raw_data" + time_condition
        
        load = SqliteDBLoad(self.db_name)
        result = load.load_sql(sql)
        
        result.columns = columns
        
        
        
        ser = Serialisation("numpy")
        result = ser.decode_dataframe(result, columns[1:])
        
        return result
    
    
    def load_table_names(self):
        """return a dictionary of all the table names and their corresponding column names"""
        con = sqlite3.connect(self.db_name)
        cur = con.cursor()
        
        ##### get all the table names
        sql = "SELECT name FROM sqlite_schema WHERE type='table'"
        cur.execute(sql)
        table_names = [table[0] for table in cur.fetchall()]
        
        ##### get all the column headers for each table
        column_headers = {}
        for table in table_names:
            sql = "PRAGMA table_info("+table+");"
            cur.execute(sql)
            column_headers[table] = [column[1] for column in cur.fetchall()]
        
        con.close()
        return column_headers


# -*- coding: utf-8 -*-
"""
copyright 2024 Peak Design
this file is part of hsr1, which is distributed under the GNU Lesser General Public License v3 (LGPL)
"""
import uuid

import pandas as pd

from hsr1.db import (SqliteDBStore,
                    Serialisation)



class SqliteStoreRaw:
    def __init__(self, driver):
        self.driver = driver
        self.serialisation = Serialisation("numpy")
    
    def store(self, dfs, deployment_metadata):
        """takes a list of dfs containing time and individual channel data"""
        table_names = ["channel_"+str(i) for i in range(len(dfs))]
        
        big_df = pd.DataFrame()
        for i, df in enumerate(dfs):
            channel = pd.DataFrame(Serialisation.listify_and_serialise_numpy(df), columns=[table_names[i]])
            channel = channel.reset_index()
            channel.columns = ["pc_time_end_measurement", table_names[i]]
            channel["pc_time_end_measurement"] = channel["pc_time_end_measurement"].astype(str)
            if len(big_df) == 0:
                big_df = channel
            else:
                big_df = big_df.merge(channel, on="pc_time_end_measurement")
        
        
        store = SqliteDBStore(self.driver)
        
        deployment_metadata = deployment_metadata.reset_index(drop=True)
        
        # TODO: more thorough check if db exists
        ##### checks if appending to a database or making new one, and if it already exists, checks to see if its the same deployment
        if self.driver.exists():
            db_load = self.driver.db_load
            store = SqliteDBStore(self.driver)
            deployment_metadata = store.match_deployment_ids(deployment_metadata, db_load)
            deployment_metadata = store.match_dataseries_ids(deployment_metadata, db_load)
        else:
            deployment_metadata["deployment_id"] = str(uuid.uuid1())
            deployment_metadata["dataseries_id"] = str(uuid.uuid1())
        
        store.store_dataframe(deployment_metadata, "deployment_metadata")
        store.store_dataframe(big_df, "raw_data")    
    


# -*- coding: utf-8 -*-
"""
copyright 2024 Peak Design
this file is part of hsr1, which is distributed under the GNU Lesser General Public License v3 (LGPL)
"""

import os

from hsr1.db import (SqliteStoreRaw,
                    SqliteLoadRaw)


class RawDbDriver:
    def __init__(self, db_name, db_type="sqlite"):
        """kwargs is here to catch old parameters that used to be useful"""
        self.db_name = db_name
        
        if db_type == "sqlite":
            self.db_store = SqliteStoreRaw(self)
            self.db_load = SqliteLoadRaw(db_name)
        else:
            raise NotImplementedError("Only sqlite databases are supported")
    
    
    def exists(self) -> bool:
        ##### TODO: move somewhere, this is sqlite specific
        """checks if a database exists and has all columns, returns bool"""
        if not os.path.isfile(self.db_name):
            print("no db file found")
            return False
        column_headings = self.db_load.load_table_names()
        if len(column_headings) >= 4:
            return True
        else:
            return False
    
    def load_table_names(self):
        return self.db_load.load_table_names()
    
    def load(self, columns:[str]=[], start_time=None, end_time=None):
        return self.db_load.load(columns, start_time, end_time)
    
    
    def store(self, dfs, deployment_metadata):
        return self.db_store.store(dfs, deployment_metadata)
    